@@ -3,7 +3,7 @@ package Hades;
 use 5.006;
 use strict;
 use warnings;
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 use Module::Generate;
 use Switch::Again qw/switch/;
 
@@ -29,8 +29,8 @@ sub run {
 		$ident =~ m/^((:.*\()|(\{)|(\[))/
 			? do {
 				my $copy = $ident;
-				$copy =~ s/\\\{|\\\}|\\\(|\\\)||\\\[|\\\]//g; # remove escaped
-				1 while ($copy =~ s/\([^()]+\)|\{[^{}]+\}|\[[^\[\]]+\]//g);
+				$copy =~ s/\\\{|\\\}|\\\(|\\\)|\\\[|\\\]//g; # remove escaped
+				1 while ($copy =~ s/\([^()]*\)|\{[^{}]*\}|\[[^\[\]]*\]//g);
 				($copy =~ m/\(|\{|\[|\)|\}|\]/) ? do {
 					$ident .= $first_char;
 				} : do {
@@ -64,6 +64,7 @@ sub run {
 							$ident .= $first_char unless $first_char =~ m/\s/;
 						};
 	}
+
 	if (scalar @lines) {
 		my $last_token;
 		for my $class (@lines) {
@@ -74,12 +75,12 @@ sub run {
 					? do { $last_token = $self->build_class_inheritance($mg, $last_token, $token); }
 					: scalar @{$token} == 1
 						? $self->build_accessor_no_arguments($mg, $token, $meta)
-						: $token->[1] =~ s/^{|}$//g
-							? $self->build_sub_no_arguments($mg, $token, $meta)
-							: $token->[0] =~ m/^(our)$/
-								? $self->build_our($mg, $token, $meta)
-								: $token->[0] =~ m/^(synopsis|abstract)$/
-									? $self->build_synopsis_or_abstract($mg, $token, $meta)
+						: $token->[0] =~ m/^(synopsis|abstract|test)$/
+							? do { my $m = "build_$1"; $self->$m($mg, $token, $meta); }
+							: $token->[1] =~ s/^{|}$//g
+								? $self->build_sub_no_arguments($mg, $token, $meta)
+								: $token->[0] =~ m/^(our)$/
+									? $self->build_our($mg, $token, $meta)
 									: $self->build_sub_or_accessor($mg, $token, $meta);
 			}
 			$self->build_new($mg, $meta);
@@ -98,7 +99,7 @@ sub build_class {
 		$mg->macro(shift @{$_}, join(' ', @{$_}) . ';') for @{$class};
 		next;
 	}
-	while ($class->[0] =~ m/^(dist|lib|author|email|version)$/) {
+	while ($class->[0] =~ m/^(dist|lib|tlib|realm|author|email|version)$/) {
 		$mg->$1($class->[1]);
 		shift @{$class}, shift @{$class};
 	}
@@ -169,10 +170,23 @@ sub build_our {
 	return $meta;
 }
 
+sub build_synopsis { goto &build_synopsis_or_abstract; }
+
+sub build_abstract { goto &build_synopsis_or_abstract; }
+
+sub build_test {
+	my ($self, $mg, $token, $meta) = @_;
+	my ($name, $content) = @{$token};
+	$content =~ s/^\{\s*|\s*\}$//g;
+	$mg->class_tests(eval $content);
+	return $meta;
+}
+
 sub build_synopsis_or_abstract {
 	my ($self, $mg, $token, $meta) = @_;
-	my $name = shift @{$token};
-	$mg->$name(join ' ', @{$token});
+	my ($name, $content) = @{$token};
+	$content =~ s/^\{\s*|\s*\}$//g;
+	$mg->$name($content);
 	return $meta;
 }
 
@@ -181,18 +195,18 @@ sub build_sub_or_accessor_attributes {
 	my @ATTR = (
 		qr/^(\:around|\:ar)/ => sub {
 			$meta->{$name}->{meta} = 'MODIFY';
-			while ($token->[0] !~ s/^\{|\}$//g) { shift @{$token} }
-			$meta->{$name}->{around} = join " ", @{$token};
+			$token->[-1] =~ s/^\{(.*)\}$/$1/sg;
+			$meta->{$name}->{around} = pop @{$token};
 		},
 		qr/^(\:after|\:a)/ => sub {
 			$meta->{$name}->{meta} = 'MODIFY';
-			while ($token->[0] !~ s/^\{|\}$//g) { shift @{$token} }
-			$meta->{$name}->{after} = join " ", @{$token};
+			$token->[-1] =~ s/^\{(.*)\}$/$1/sg;
+			$meta->{$name}->{after} = pop @{$token};
 		},
 		qr/^(\:before|\:b)/ => sub {
 			$meta->{$name}->{meta} = 'MODIFY';
-			while ($token->[0] !~ s/^\{|\}$//g) { shift @{$token} }
-			$meta->{$name}->{before} = join " ", @{$token};
+			$token->[-1] =~ s/^\{(.*)\}$/$1/sg;
+			$meta->{$name}->{before} = pop @{$token};
 		},
 		qr/^(\:clearer|\:c)$/ => sub {
 			$meta->{$name}->{clearer} = 1;
@@ -219,6 +233,16 @@ sub build_sub_or_accessor_attributes {
 				}->{default} = $value;
 			}
 		},
+		qr/^(\:example)/ => sub {
+			my $value = shift;
+			$value =~ s/^\:example\(\s*(.*)\s*\)$/$1/sg;
+			$meta->{$name}->{example} =  $value;
+		},
+		qr/^(\:pod)/ => sub {
+			my $value = shift;
+			$value =~ s/^:pod\(\s*(.*)\s*\)$/$1/sg;
+			$meta->{$name}->{pod} =  $value;
+		},
 		qr/^(\:private|\:p)$/ => sub {
 			$meta->{$name}->{private} = 1;
 		},
@@ -251,7 +275,7 @@ sub build_sub_or_accessor_attributes {
 		qr/^(\{)/ => sub {
 			my $value = shift;
 			$value =~ s/^\{|\}$//g;
-			$meta->{$name}->{meta} = 'METHOD';
+			$meta->{$name}->{meta} = 'METHOD' unless $meta->{$name}->{meta} eq 'MODIFY';
 			$meta->{$name}->{code} = $value;
 		},
 		qr/^(\%|\$|\@|\&)/ => sub {
@@ -298,6 +322,13 @@ sub build_accessor {
 		return \$self->{$name};
 	}|;
 	$mg->accessor($name)->code($code)->clear_tests->test($self->build_tests($name, $meta->{$name}));
+	$meta->{$name}->{$_} && $mg->$_($self->replace_pe_string($meta->{$name}->{$_}, $name)) for qw/pod example/;
+}
+
+sub replace_pe_string {
+	my ($self, $str, $name) = @_;
+	$str =~ s/\$name/$name/g;
+	return $str;
 }
 
 sub build_modify {
@@ -311,6 +342,7 @@ sub build_modify {
 		return wantarray ? \@res : \$res[0];
 	}|;
 	$mg->sub($name)->code($code)->pod(qq|call $name method.|)->test($self->build_tests($name, $meta->{$name}));
+	$meta->{$name}->{$_} && $mg->$_($self->replace_pe_string($meta->{$name}->{$_}, $name)) for qw/pod example/;
 }
 
 sub build_sub {
@@ -356,6 +388,7 @@ sub build_sub {
 		->pod(qq|call $name method. Expects $params_explanation.|)
 		->example($example)
 		->test($self->build_tests($name, $meta->{$name}));
+	$meta->{$name}->{$_} && $mg->$_($self->replace_pe_string($meta->{$name}->{$_}, $name)) for qw/pod example/;
 }
 
 sub build_clearer {
@@ -1032,7 +1065,7 @@ Hades - Less is more, more is less!
 
 =head1 VERSION
 
-Version 0.12
+Version 0.13
 
 =cut
 
@@ -1198,6 +1231,30 @@ Declare a new class.
 
 =cut
 
+=head3 Abstract
+
+Declare the classes Abstract.
+
+	Kosmos {
+		abstract { Afti einai i perilipsi }
+	}
+
+=cut
+
+=head3 Synopsis
+
+Declare the classes Synopsis.
+
+	Kosmos {
+		synopsis {
+			Schetika me ton Kosmos
+				
+				Kosmos->new;
+		}
+	}
+
+=cut
+
 =head3 Inheritance
 
 =cut
@@ -1244,6 +1301,21 @@ Declare modules that should be included in the class.
 
 =cut
 
+=head3 Test
+
+Declare the classes additional tests.
+
+	Kosmos {
+		test {
+			[ 
+				['ok', 'my $obj = Kosmos->new'],
+				['is', '$obj->dokimi', undef]
+			]
+		}
+	}
+
+=cut
+
 =head2 Compile phase
 
 =cut
